function [lambdas,noise_dat] = PoissonLambda(zAs,specdat,varargin) 
% function [lambdas,noise_dat] = PoissonLambda(zAs,specdat,varargin) 
% handle 3 or more function basis sets
% inputs:
%  zAs: a vector, As(1) + 1i*As(2) or an nbasis column matrix 
%      specdat: a spectrum data struct for incident spectrum with fields
%        egy: x-ray energies
%        specnum: the photon number spectrum
%        zmus: the basis materials attenuation coefficients at specified energies. mus_1(E) + 1i*mus_2(E)
%       Ethreshold: (keV) interbin separation energies-- this is preferred method to specify bins. Will be used if both
%           this and idx_threshold present
%        idx_threshold: indices into the spectrum array for inter-bin energies
%    fwhm: (optional = 0) fraction of input energy range to model effect of finite energy resolution
% outputs:
%   lambdas: the integrated photon counts for each zA values for each spectrum arranged along rows
%   noise_dat: a struct with fields
% REA 4/20/10
% 9/16/10 handle 3 or more function basis sets

nreqargs = 2;
assert(nargin>=nreqargs);

fwhm = 0;

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'no_qs';          no_qs = true;
     case 'means_only';          means_only = true;
     case 'ntrials';         ntrials=varargin{i+1}; assert(ntrials>=1);    i=i+1;
     case 'fwhm';         fwhm=varargin{i+1}; assert(fwhm>=0);    i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

if isreal(zAs) % i.e. not complex
%    assert(size(zAs,2)>2);
   assert(isfield(specdat,'mus'));
   assert(size(specdat.mus,2)==size(zAs,2));
   iscmplx = false;
else
    zAs = zAs(:); % convert to column vector
    npnts = numel(zAs);
    iscmplx = true;
       % test for mus and create zmus if necessary
    if ~isfield(specdat,'zmus')
        specdat.zmus = zcomp(specdat.mus);
    end
end

dE = specdat.egys(2) - specdat.egys(1);
negys = numel(specdat.egys);


       % compute the indexes into the egy array for the N bins
if isfield(specdat,'Ethreshold')
   nbins = numel(specdat.Ethreshold) + 1;
   idx_threshold = zeros(nbins-1,1);
   for k = 1:(nbins-1)
      [minval,idx] = min(abs(specdat.Ethreshold(k) - specdat.egys));
      idx_threshold(k) = idx;
   end
   specdat.idx_threshold = idx_threshold;
end

idx_threshold = specdat.idx_threshold;       
nbins = numel(idx_threshold)+1;       
idxs = cell(nbins,1);
bin_edges = [0; idx_threshold(:);negys];
lambdas = zeros(size(zAs,1),nbins);
if fwhm>0 % handle case of finite energy resolution
    spec_bins = cell(nbins,1); 
    hzero = zeros(negys,1); % prep for convolution to handle finite energy resolution
    dev = fwhm*range(specdat.egys)/sqrt(2*log(2))/2;
    hgauss_spec = fft(circshift(normpdf(specdat.egys,mean(specdat.egys),dev),-round(negys/2)))*dE; % note transpose for column vector
    sp = specdat; % make local copy
    for k = 1:nbins
        h = hzero;
        h( (bin_edges(k)+1):bin_edges(k+1)) = 1;
        sp.specnum = ifft(fft(specdat.specnum.*h).*hgauss_spec);
        spec_bins{k} = sp;
        lambdas(:,k) = flambda(zAs,sp,iscmplx)'; % note transpose, want column vector
        idxs{k} = idx;
    end
else
    for k = 1:nbins
        idx = (bin_edges(k)+1):bin_edges(k+1);
        sp.specnum = specdat.specnum(idx);
        sp.egys = specdat.egys(idx);
        if iscmplx
            sp.zmus = specdat.zmus(idx);
        else
            sp.mus = specdat.mus(idx,:);
        end
        lambdas(:,k) = flambda(zAs,sp,iscmplx)'; % note transpose, want column vector
        idxs{k} = idx;
    end
end

if nargout>1
    noise_dat.specdat = specdat;
    noise_dat.idxs = idxs;
    noise_dat.bin_edges = bin_edges;
    noise_dat.mean_vals = lambdas;
    if fwhm>0
       noise_dat.spec_bins = spec_bins;
    end
    
       % compute Ebar and E2bar-- 
       % 9/17/10 has bug so comment out since do not use
%     lambdas_all = flambda(zAs,specdat,iscmplx);
%     noise_dat.lambdas_all = lambdas_all;
%     Tr_all = exp(-[real(specdat.zmus(:)) imag(specdat.zmus(:))]*[real(zAs(:)).'; imag(zAs(:)).']);
%     
%     noise_dat.Ebar = sum( (specdat.egys(:).*specdat.specnum(:))' * Tr_all,1)./lambdas_all(:)';
%     noise_dat.E2bar = sum( ((specdat.egys(:).^2).*specdat.specnum(:))' * Tr_all,1)./lambdas_all(:)';
end
   
   
%---------- LOCAL FUNCTIONS -----------
  % define local function for lambda
function lambda = flambda (zAs,specdat,iscmplx)
   % returns the mean values of photon counts
  if iscmplx
    lambda = sum(  specdat.specnum(:)' *exp( -zcomp(specdat.zmus(:))*zcomp(zAs(:))' ) ,1 ); 
  else
    lambda = sum( specdat.specnum(:)' *exp(-specdat.mus*zAs'),1 ); 
  end
   