function [M,mdat] = mubarNbin(specdat,zAs,varargin)
% function M = mubarNbin(specdat,zAs)
% computes effective mu matrix for specified zAs thicknesses. 
%  inputs:
%    specdat: spectrum struct with fields
%       specnum: number spectrum
%       zmus: mus_1(E) + 1i*mus_2(E)
%       idx_threshold: the indices for interbin energies
%    zAs: As(:,1) + 1i*As(:,2)
%  outputs:
%    M: an nspect,2,numel(zAs) matrix of effective mus in bins specified by specdat.idx_threshold
%    mdat: debugging info struct. See code for fields (not implemented)
%  REA 4/20-2011-Aug-5 10:50

idxs4bins = [];
nreqargs = 2;
assert(nargin>=nreqargs);

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'no_qs';          no_qs = true;
     case 'idxs4bins';         idxs4bins=varargin{i+1}; assert(iscell(idxs4bins));    i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

if ~isfield(specdat,'zmus')
    assert(size(specdat.mus,2)==2);
    specdat.zmus = zcomp(specdat.mus);
end


if isfield(specdat,'Ethreshold')
   if isfield(specdat,'idx_threshold')
       warning('mubarNbin: overriding idx_threshold with Ethreshold');
   end
   nbins = numel(specdat.Ethreshold) + 1;
   idx_threshold = zeros(nbins-1,1);
   for k = 1:(nbins-1)
      [minval,idx] = min(abs(specdat.Ethreshold(k) - specdat.egys));
      idx_threshold(k) = idx;
   end
   specdat.idx_threshold = idx_threshold;
end

% compute the indexes into the egy array for the N bins
idx_threshold = specdat.idx_threshold;       
nspect = numel(idx_threshold)+1;
       
if isempty(idxs4bins)
    bin_edges = [0; idx_threshold(:);numel(specdat.egys)];
    idxs = cell(nspect,1);
    for k = 1:nspect
        idxs{k}= (bin_edges(k)+1):bin_edges(k+1);
    end
else % idxs were entered as a parameter
   idxs = idxs4bins;
end
nthicknesses = numel(zAs);

M = zeros(nspect,2,nthicknesses);
lambdas = zeros(nspect,1);
dE = specdat.egys(2) -  specdat.egys(1) ;
for kspect = 1:nspect
    idx = idxs{kspect};
    sp = specdat.specnum(idx)';
    lambdas(kspect) = sum(sp)*dE;
    zmus = specdat.zmus(idx).';
    egys = specdat.egys(idx);
    for kt = 1:nthicknesses
        zA = zAs(kt);
        Transmission_E = exp(-[real(zmus(:)) imag(zmus(:))]*[real(zA); imag(zA)])';
        spec_transmitted = sp.*Transmission_E;
        sp_norm = spec_transmitted./ repmat(trapz(egys,spec_transmitted),1,numel(idx));
        mz = trapz(egys,zmus.*sp_norm,2);
        M(kspect,:,kt) = [real(mz) imag(mz)];
    end
end

if nargout>1
    sp = specdat.specnum;
    zmus = specdat.zmus; % for whole spectrum
    Transmission_E = exp(-[real(zmus(:)) imag(zmus(:))]*[real(zA); imag(zA)])';
    spec_transmitted = sp(:).*Transmission_E(:);
    lambda = sum(spec_transmitted(:))*dE;
    sp_norm = spec_transmitted/lambda;
    specT = specdat;
    specT.specnum = sp_norm;
    specT.dE = dE;
    mdat.lambdas = lambdas;
    mdat.specT = specT;
    mz = sum(zmus(:).*sp_norm(:))*dE;
    mdat.lambda = lambda;
    mdat.mz = mz;
end